@@ -25,6 +25,8 @@
 #ifndef __GST_UTILS_H__
 #define __GST_UTILS_H__
 
+#include <gst/gstconfig.h>
+
 #include <glib.h>
 #include <gst/gstbin.h>
 
@@ -114,7 +116,7 @@ GType									\
 type_as_function ## _get_type (void)					\
 {									\
   static GType object_type = 0;						\
-  if (object_type == 0) {						\
+  if (G_UNLIKELY (object_type == 0)) {					\
     static const GTypeInfo object_info = {				\
       sizeof (type ## Class),						\
       type_as_function ## _base_init,					\
@@ -187,7 +189,7 @@ static gboolean interface_as_function ## _supported (type *object, GType iface_t
 static void                                                             \
 type_as_function ## _implements_interface_init (GstImplementsInterfaceClass *klass)     \
 {                                                                       \
-  klass->supported = (gpointer)interface_as_function ## _supported;     \
+  klass->supported = (gboolean (*)(GstImplementsInterface*, GType))interface_as_function ## _supported;     \
 }                                                                       \
                                                                         \
 static void                                                             \
@@ -531,6 +533,12 @@ void                    gst_element_unlink_pads         (GstElement *src, const
 gboolean		gst_element_link_pads_filtered	(GstElement * src, const gchar * srcpadname,
                                                          GstElement * dest, const gchar * destpadname,
                                                          GstCaps *filter);
+
+gboolean        gst_element_seek_simple (GstElement   *element,
+                                         GstFormat     format,
+                                         GstSeekFlags  seek_flags,
+                                         gint64        seek_pos);
+
 /* util elementfactory functions */
 gboolean		gst_element_factory_can_src_caps(GstElementFactory *factory, const GstCaps *caps);
 gboolean		gst_element_factory_can_sink_caps(GstElementFactory *factory, const GstCaps *caps);
@@ -565,6 +573,13 @@ gboolean                gst_pad_query_duration          (GstPad *pad, GstFormat
 gboolean                gst_pad_query_convert           (GstPad *pad, GstFormat src_format, gint64 src_val,
 		                                         GstFormat *dest_format, gint64 *dest_val);
 
+gboolean                gst_pad_query_peer_position     (GstPad *pad, GstFormat *format,
+		                                         gint64 *cur);
+gboolean                gst_pad_query_peer_duration     (GstPad *pad, GstFormat *format,
+		                                         gint64 *duration);
+gboolean                gst_pad_query_peer_convert      (GstPad *pad, GstFormat src_format, gint64 src_val,
+		                                         GstFormat *dest_format, gint64 *dest_val);
+
 /* bin functions */
 void                    gst_bin_add_many                (GstBin *bin, GstElement *element_1, ...) G_GNUC_NULL_TERMINATED;
 void                    gst_bin_remove_many             (GstBin *bin, GstElement *element_1, ...) G_GNUC_NULL_TERMINATED;
@@ -599,10 +614,18 @@ void			gst_element_found_tags_for_pad	(GstElement * element,
 void			gst_element_found_tags		(GstElement * element,
 							 GstTagList * list);
 
+#ifndef GST_DISABLE_PARSE
 /* parse utility functions */
 GstElement *            gst_parse_bin_from_description  (const gchar * bin_description,
                                                          gboolean ghost_unconnected_pads,
                                                          GError ** err);
+#else /* GST_DISABLE_PARSE */
+
+#if defined _GNUC_ && _GNUC_ >= 3
+#pragma GCC poison gst_parse_bin_from_description 
+#endif
+
+#endif
 
 G_END_DECLS
 
