@@ -28,7 +28,8 @@
  *
  * Data is queued till max_level buffers have been stored. Any subsequent
  * buffers sent to this filter will block until free space becomes available in
- * the buffer. The queue is typically used in conjunction with a thread.
+ * the buffer. The queue will create a new thread on the source pad to
+ * decouple the processing on sink and source pad.
  *
  * You can query how many buffers are queued with the level argument.
  *
@@ -75,7 +76,7 @@ GST_DEBUG_CATEGORY_STATIC (queue_dataflow);
                       queue->max_size.time, \
                       queue->queue->length)
 
-static GstElementDetails gst_queue_details = GST_ELEMENT_DETAILS ("Queue",
+static const GstElementDetails gst_queue_details = GST_ELEMENT_DETAILS ("Queue",
     "Generic",
     "Simple data queue",
     "Erik Walthinsen <omega@cse.ogi.edu>");
@@ -145,6 +146,7 @@ static void gst_queue_get_property (GObject * object,
 static GstFlowReturn gst_queue_chain (GstPad * pad, GstBuffer * buffer);
 static GstFlowReturn gst_queue_bufferalloc (GstPad * pad, guint64 offset,
     guint size, GstCaps * caps, GstBuffer ** buf);
+static gboolean gst_queue_acceptcaps (GstPad * pad, GstCaps * caps);
 static gboolean gst_queue_push_one (GstQueue * queue);
 static void gst_queue_loop (GstPad * pad);
 
@@ -172,7 +174,7 @@ static GType
 queue_leaky_get_type (void)
 {
   static GType queue_leaky_type = 0;
-  static GEnumValue queue_leaky[] = {
+  static const GEnumValue queue_leaky[] = {
     {GST_QUEUE_NO_LEAK, "Not Leaky", "no"},
     {GST_QUEUE_LEAK_UPSTREAM, "Leaky on Upstream", "upstream"},
     {GST_QUEUE_LEAK_DOWNSTREAM, "Leaky on Downstream", "downstream"},
@@ -356,6 +358,8 @@ gst_queue_init (GstQueue * queue)
       GST_DEBUG_FUNCPTR (gst_queue_src_activate_push));
   gst_pad_set_link_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_link_src));
+  gst_pad_set_acceptcaps_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_acceptcaps));
   gst_pad_set_getcaps_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_event_function (queue->srcpad,
@@ -373,6 +377,9 @@ gst_queue_init (GstQueue * queue)
   queue->min_threshold.buffers = 0;     /* no threshold */
   queue->min_threshold.bytes = 0;       /* no threshold */
   queue->min_threshold.time = 0;        /* no threshold */
+  queue->orig_min_threshold.buffers = 0;
+  queue->orig_min_threshold.bytes = 0;
+  queue->orig_min_threshold.time = 0;
 
   queue->leaky = GST_QUEUE_NO_LEAK;
   queue->srcresult = GST_FLOW_WRONG_STATE;
@@ -472,13 +479,19 @@ gst_queue_bufferalloc (GstPad * pad, guint64 offset, guint size, GstCaps * caps,
 
   queue = GST_QUEUE (GST_PAD_PARENT (pad));
 
-  result =
-      gst_pad_alloc_buffer_and_set_caps (queue->srcpad, offset, size, caps,
-      buf);
+  /* Forward to src pad, without setting caps on the src pad */
+  result = gst_pad_alloc_buffer (queue->srcpad, offset, size, caps, buf);
 
   return result;
 }
 
+static gboolean
+gst_queue_acceptcaps (GstPad * pad, GstCaps * caps)
+{
+  /* The only time our acceptcaps method should be called is on the srcpad
+   * when we push a buffer, in which case we always accept those caps */
+  return TRUE;
+}
 
 static void
 gst_queue_locked_flush (GstQueue * queue)
@@ -493,6 +506,9 @@ gst_queue_locked_flush (GstQueue * queue)
   queue->cur_level.buffers = 0;
   queue->cur_level.bytes = 0;
   queue->cur_level.time = 0;
+  queue->min_threshold.buffers = queue->orig_min_threshold.buffers;
+  queue->min_threshold.bytes = queue->orig_min_threshold.bytes;
+  queue->min_threshold.time = queue->orig_min_threshold.time;
 
   /* we deleted something... */
   g_cond_signal (queue->item_del);
@@ -520,7 +536,8 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
       g_cond_signal (queue->item_del);
       GST_QUEUE_MUTEX_UNLOCK (queue);
 
-      /* make sure it pauses */
+      /* make sure it pauses, this should happen since we sent
+       * flush_start downstream. */
       gst_pad_pause_task (queue->srcpad);
       GST_CAT_LOG_OBJECT (queue_dataflow, queue, "loop stopped");
       goto done;
@@ -560,10 +577,11 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
 
   GST_QUEUE_MUTEX_LOCK (queue);
   if (have_eos) {
-    /* FIXME, abusing the cur_level */
-    queue->cur_level.buffers = queue->max_size.buffers;
-    queue->cur_level.bytes = queue->max_size.bytes;
-    queue->cur_level.time = queue->max_size.time;
+    /* Zero the thresholds, this makes sure the queue is completely
+     * filled and we can read all data from the queue. */
+    queue->min_threshold.buffers = 0;
+    queue->min_threshold.bytes = 0;
+    queue->min_threshold.time = 0;
   }
   g_queue_push_tail (queue->queue, event);
   g_cond_signal (queue->item_add);
@@ -673,6 +691,7 @@ gst_queue_chain (GstPad * pad, GstBuffer * buffer)
       case GST_QUEUE_NO_LEAK:
         STATUS (queue, pad, "pre-full wait");
 
+        /* we recheck, the signal could have changed the thresholds */
         while (gst_queue_is_filled (queue)) {
           STATUS (queue, pad,
               "waiting for item_del signal from thread using qlock");
@@ -767,12 +786,7 @@ gst_queue_push_one (GstQueue * queue)
       flowname = gst_flow_get_name (result);
 
       queue->srcresult = result;
-      if (GST_FLOW_IS_FATAL (result)) {
-        GST_ELEMENT_ERROR (queue, STREAM, FAILED,
-            (_("Internal data flow error.")),
-            ("streaming stopped, reason %s", flowname));
-        gst_pad_push_event (queue->srcpad, gst_event_new_eos ());
-      }
+
       GST_DEBUG_OBJECT (queue, "pausing queue, reason %s", flowname);
       gst_pad_pause_task (queue->srcpad);
     }
@@ -831,6 +845,7 @@ restart:
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
 
     STATUS (queue, pad, "pre-empty wait");
+    /* we recheck, the signal could have changed the thresholds */
     while (gst_queue_is_empty (queue)) {
       STATUS (queue, pad, "waiting for item_add");
 
@@ -1036,6 +1051,19 @@ gst_queue_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
+/* changing the capacity of the queue must wake up
+ * the _chain function, it might have more room now
+ * to store the buffer/event in the queue */
+#define QUEUE_CAPACITY_CHANGE(q)\
+  g_cond_signal (queue->item_del);
+
+/* Changing the minimum required fill level must
+ * wake up the _loop function as it might now
+ * be able to preceed.
+ */
+#define QUEUE_THRESHOLD_CHANGE(q)\
+  g_cond_signal (queue->item_add);
+
 static void
 gst_queue_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec)
@@ -1049,21 +1077,30 @@ gst_queue_set_property (GObject * object,
   switch (prop_id) {
     case ARG_MAX_SIZE_BYTES:
       queue->max_size.bytes = g_value_get_uint (value);
+      QUEUE_CAPACITY_CHANGE (queue);
       break;
     case ARG_MAX_SIZE_BUFFERS:
       queue->max_size.buffers = g_value_get_uint (value);
+      QUEUE_CAPACITY_CHANGE (queue);
       break;
     case ARG_MAX_SIZE_TIME:
       queue->max_size.time = g_value_get_uint64 (value);
+      QUEUE_CAPACITY_CHANGE (queue);
       break;
     case ARG_MIN_THRESHOLD_BYTES:
       queue->min_threshold.bytes = g_value_get_uint (value);
+      queue->orig_min_threshold.bytes = queue->min_threshold.bytes;
+      QUEUE_THRESHOLD_CHANGE (queue);
       break;
     case ARG_MIN_THRESHOLD_BUFFERS:
       queue->min_threshold.buffers = g_value_get_uint (value);
+      queue->orig_min_threshold.buffers = queue->min_threshold.buffers;
+      QUEUE_THRESHOLD_CHANGE (queue);
       break;
     case ARG_MIN_THRESHOLD_TIME:
       queue->min_threshold.time = g_value_get_uint64 (value);
+      queue->orig_min_threshold.time = queue->min_threshold.time;
+      QUEUE_THRESHOLD_CHANGE (queue);
       break;
     case ARG_LEAKY:
       queue->leaky = g_value_get_enum (value);
